:
#	@(#) prep.sup.sh 23.2 91/12/10 
#
#	Copyright (C) 1989-1991 The Santa Cruz Operation, Inc.
#		All Rights Reserved.
#	The information in this file is provided for the exclusive use of
#	the licensees of The Santa Cruz Operation, Inc.  Such users have the
#	right to use, modify, and incorporate this code into other products
#	for purposes authorized by the license agreement provided they include
#	this notice and the associated copyright notice with any such product.
#	The information in this file is provided "AS IS" without warranty.
#
PATH=/bin:/usr/bin:/etc
prd=sup.os

# Return code definitions 
: ${OK=0} ${FAIL=1} ${STOP=10} ${HALT=11}

# set up default values
quit=quit				# q option printed after prompt
tmp=/tmp/rts$$				# temporary file name 
retn="\007\nand press <RETURN> "	# commonly used in prompts

# Functions definitions

trap 'cleanup 1' 1 2 3 15

# local_menu()
# Usage : local_menu key directory filename
# Dependencies : intlecho(), basename()
# Note :
#	The directory is the relative or absolute
#	pathname where the localised 
#	directories are kept.
#	The filename is the name of the message file
#	under the localised directories.
#	It will set up the global TRANSFILE on return
#	with the user chosen message file.

local_menu() {

	TRANSDIR=$2

	# if LANG variable defined use it.
	# LANG can be of "per language" as in LANG=english
	# The other fields will be taken from the defaults in
	# /etc/default/lang.
	# see - Intl OS System Guide, Administering International
	# Environment.

	[ "$LANG" ] && {
		# has LANG got a underscore
		echo $LANG | grep _ >/dev/null 2>&1 && {
			# yes
			lang=`echo $LANG | sed -n "s/^\(.*\)\..*$/\1/p"`
		} || {
			# no, fill in the field from /etc/default/lang
			lang=`sed -n "s/^LANG=\(${LANG}.*\)\..*$/\1/p" /etc/default/lang`
		}
		
		[ -f $TRANSDIR/$lang/$3 ] && {
			TRANSFILE=$TRANSDIR/$lang/$3
			return 0
		}
	}
	
	[ -f /etc/default/lang ] && {
		lang=`sed -n "s/^LANG=\(.*\)\..*$/\1/p" /etc/default/lang`
		for i in $lang
		do
			[ -f $TRANSDIR/$i/$3 ] && {
				TRANSFILE=$TRANSDIR/$i/$3
				return 0
			}
		done
	}

	count=0

	for i in $TRANSDIR/*
	do
		[ -f $i/$3 ] || continue
		heading=`intlecho "$1" "$i/$3"`
		[ "$heading" ] || continue
		count=`expr $count + 1`
		eval language$count=\$heading
		eval option$count=`basename $i`
	done

	# if count is equal to 0, then the message
	# files do not contain the key
	[ $count -eq 0 ] && return 1

	num_lang=$count

	[ "$num_lang" -eq "1" ] && {
		TRANSFILE=$TRANSDIR/$option1/$3
		return 0
	}

	count=0

	while [ "$count" -lt "$num_lang" ]
	do
		count=`expr $count + 1`
		eval echo "$count.  \$language$count" >> /tmp/$$
	done

	while true
	do
		cat /tmp/$$
		echo -n "? "
		read answer

		case $answer in
		[1-9]|[1-9][0-9])
				  [ "$answer" -le "$num_lang" ] && {
					eval TRANSFILE=\$TRANSDIR/\$option$answer/\$3
					return 0 
				  }
					;;
		esac

	done

}


# intlecho()
# usage : intlecho key message_file
# Note  :  
#	  Absolute or relative pathname of the message file.
#         All the lines with the key will be displayed.

intlecho() {

echo "s!^$1\(.*\)\$!\1!p" > ${tmp}echo
sed -n -f ${tmp}echo $2
rm -f ${tmp}echo

}

# getyn()
# Usage : getyn message
# Prompt for yes or no - returns non-zero for no
getyn() {

	while echo "$* ($GETYNy/$GETYNn) \c" >&2
	do	read yn rest
		case $yn in
		$GETYNy|$GETYNY)
			return 0
					;;
		$GETYNn|$GETYNN)
			return 1
					;;
		esac
		# Please answer y or n
		echo "$GETYN1 $GETYNy/$GETYNn " >&2
	done
}

# error()
# Usage : error
# Print an error message
error() {
	echo "$ERROR1 $*" >&2
	return 1
}
# cleanup()
# Usage : cleanup
# Remove temp files and exit with the status passed as argument
cleanup() {
	trap '' 1 2 3 15
	cd /
	[ "$tmp" ] && rm -f $tmp*
	rm -rf /tmp/perms/prep
	exit $1
}
# prompt()
# Usage : prompt
# Prompt with mesg, return non-zero on q. Also allows setting -x and +x,
# shell escapes and the passing of default values
prompt() {
	mesg=$1
	while	echo "\n${mesg}$PROMPT1 $quit: \c" >&2
	do	read cmd
		case $cmd in
		+x|-x)	set $cmd					;;
		$PROMPT2|$PROMPT3)	return 1			;;
		!*)	eval `expr "$cmd" : "!\(.*\)"`			;;
		"")	# If there is an second argument use it
			# as the default else loop until 'cmd' is set
			[ "$2" ] && { 
				cmd=$2
				return 0
			}
			: continue
			;;
		*)	return 0					;;
		esac
	done
}

#
# setenv()
#
# Usage: setenv prdcode
#
# - if we're running SCO Unix (ie /etc/perms/inst exists), sets _os
#   by inspecting /etc/perms/inst, and _cpu and _osrel by inspecting the
#   output of uname(C). Inserts `80' into _cpu if we're running SCO Unix
# - sets _prdtyp by inspecting /tmp/perms/<prdcode>
# - returns 0 if we're running SCO Unix, 1 otherwise
#

setenv() {

	_prdcode=$1
	_currdir=`pwd`
	cd /

	[ -f /etc/perms/inst ] && {

		#
		# We are running SCO Xenix/Unix - find out which
		#

		_retcode=0

		_os=`sed -n '/^#prd=/s/#prd=//p' /etc/perms/inst`
		__cpu=`uname -p`
		_osrel=`uname -r`
		case $_os in 
		xos)	_cpu=$__cpu
			_retcode=1
			;;
		unixos)	case $__cpu in
			i286)	_cpu=i80286	;;
			i386)	_cpu=i80386	;;
			esac
		esac
	} || {

		_retcode=1

		_os=
		_cpu=
		_osrel=
	}

	_prdtyp=`sed -n '/^#typ=/s/#typ=//p' /tmp/perms/$1`

	cd $_currdir
	return $_retcode
}

#
# checkenv()
#
# - ensure that the product being installed matches the OS/cpu
#
# - the generic matrix is as follows:
#
#		    |	  Product
#		    |	286	386
#	------------+--------------
#		286 |   ok	stop
#	CPU	    |
#		386 |   warn	ok
#
# - returns 0 if ok or accepted warning, 1 otherwise
#

checkenv() {

	case $_cpu in
	i80286)	case $_prdtyp in
		n286|k286|u286)
			# i80286 product onto i80286 OS - ok
			return 0	;;
		*)	# i80386 product onto i80286 OS - stop
			eval echo $CKENV1
			return 1	;;
		esac
		;;
	i80386) case $_prdtyp in
		n386|k386|u386)
			# i80386 product onto i80386 OS - ok
			return 0	;;
		*)	# i80286 product onto i80386 OS - warn
			eval echo $CKENV2
			getyn "$CKENV3" && return 0
			return 1	;;
		esac
		;;
	esac
}

#
# checkrel()
#
# - ensure that this product release can install on this OS release
#
# - return $OK or $FAIL
#

checkrel() {
	case $_os in
	xos)
		case $_osrel in
		2.3.[4-9])			return $OK	;;
		2.[4-9].[0-9])			return $OK	;;
		*)				eval error "$CKREL1"
						return $FAIL	;;
		esac
		;;
	unixos)
		case $_osrel in

		3.[2-9].[0-9])			return $OK	;;
		3.[2-9])			return $OK	;;
		*)				eval error "$CKREL1"
						return $FAIL	;;
		esac
		;;
	esac
}

# check_single() 
# Usage : check_single
# check for single user mode
#
# Under Unix use "who -r", in Xenix we count how many logins show up on who.
#
check_single() {
	[ "$_os" = "xos" ] && {
		[ "`who | wc -w`" -eq 0 ] || {
			echo "$CHECK1"
			getyn "$CHECK2" || exit $STOP
		}
	} || {
		[ "`who -r | awk '{ print $3 }'`" != "S" ] && {
			echo "$CHECK1"
			getyn "$CHECK2" || exit $STOP
		}
	}
	return $OK
}

# patch_permsfile()
# Usage: patch_permsfile
# This function patches the permsfile if we're running on a Xenix
# system, because it whines about the gid and uid values being different
# than those listed in /etc/group and /etc/passwd.
#
patch_permsfile() {
	[ "$_os" = "xos" ] && {
		ed - ./tmp/perms/sup.os <<END_PATCH
/# User id's:/-1,/# Fields are:/-2c
#
# User id's:
#
uid	root		0
uid	bin		3
uid	uucp		4
uid	asg		6
uid	sysinfo		10
uid	network		12
uid	lp		14
uid	dos		16
#
# Group id's:
#
gid	root		0
gid	bin		3
gid	uucp		4
gid	asg		6
gid	sysinfo		10
gid	network		12
.
w
q
END_PATCH
	}
}

# testpkg()
# Usage : testpkg pkg descr perm
# Test to see if a package is installed and prompt to install it if not.
# Dependencies: getyn(), error() and custom
# Notes: Return 1 if package cannot be installed.
#	 Assumes that the current directory is appropriate for fixperm to work.
#	 The function requires the following parameters
#		The package name in upper case
#		A description of the package
#		The permlist to be used (optional, uses $prdperm if no permlist
#			is passed)
# 	 Assumes that settools has already been run to set $fixperm and $prdperm
#
testpkg() { 
	fixperm=./etc/fixperm
	prdperm=./etc/perms/ext
	_pkg=$1 _descr=$2 _perm=${3:-${prdperm?}}
	until ${fixperm?} -i -d$_pkg ${_perm?} > /dev/null 2>&1
	do
		case $? in
	    	0)	unset _descr _pkg; return 0			  ;;
	    	4|6)	echo "\n$_descr $TESTPKG1." >&2		  ;;
	    	5)	echo "\n$_descr $TESTPKG2." >&2 ;;
	    	*)	if [ -f ${_perm?} ]; then
				error "$TESTPKG3 $_descr $TESTPKG4."
				return 1
			else
				echo "\n$_descr $TESTPKG1." >&2
			fi
			;;
	    	esac
		getyn "$TESTPKG5" &&
			/bin/sh -c "/etc/custom -o -i $_pkg" || return 1
	done
	unset _descr _pkg
	return 0
}

# main()
# must be at root
cd /
local_menu "MENU=" "./tmp/perms/prep" "prep.message" || cleanup $FAIL
. $TRANSFILE

LANG=english_us.ascii
export LANG

setenv sup.os
checkenv || cleanup $FAIL
checkrel || cleanup $FAIL
check_single || cleanup $FAIL
# If we're running XENIX, we must have the INTL package installed.
#
[ "$_os" = "xos" ] && {
	testpkg INTL "International Character set support" || {
		echo "$CHECKINTL1"
		cleanup $FAIL
	}
}
patch_permsfile
cleanup $OK
